/******************************************************************************
 *   Copyright (C) 2008 by Oliver Bock                                        *
 *   oliver.bock[AT]aei.mpg.de                                                *
 *                                                                            *
 *   This file is part of PulsatingScience.                                   *
 *                                                                            *
 *   PulsatingScience is free software: you can redistribute it and/or modify *
 *   it under the terms of the GNU General Public License as published        *
 *   by the Free Software Foundation, version 3 of the License.               *
 *                                                                            *
 *   PulsatingScience is distributed in the hope that it will be useful,      *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             *
 *   GNU General Public License for more details.                             *
 *                                                                            *
 *   You should have received a copy of the GNU General Public License        *
 *   along with PulsatingScience. If not, see <http://www.gnu.org/licenses/>. *
 *                                                                            *
 ******************************************************************************/

#ifndef PULSARANIMATIONWIDGET_H_
#define PULSARANIMATIONWIDGET_H_

#include <cmath>

#include <QGLWidget>
#include <QTimer>
#include <QMouseEvent>

#define PI 3.14159265


class PulsarAnimationWidget : public QGLWidget
{
	Q_OBJECT

public:
	PulsarAnimationWidget(QWidget *parent);
	virtual ~PulsarAnimationWidget();

	void setFramePerSecond(const unsigned int fps);
	void setCompanionMass(const float mass);
	void setPulsarMass(const float mass);
	void setPulsarSpinFrequency(const float frequency);
	void setPulsarSpinAxisInclination(const int degrees);
	void setPulsarMagneticAxisInclination(const int degrees);
	void setPulsarSemiMajorAxis(const float length);

public slots:
	void runAnimation();
	void pauseAnimation();
	void stopAnimation();

	void updateFrame();

	void showOrbits(bool enabled);
	void showRotationAxes(bool enabled);

signals:
	void pulsarSemiMajorAxisUpdated(double value);
	void pulseProfileUpdated(const QVector<float>& data);
	void pulsarAnimationStep(float stepSize);
	void pulsarAnimationReset();

private:
    void initializeGL();
    void resizeGL(int w, int h);
    void paintGL();

    void mousePressEvent(QMouseEvent *event);
    void mouseMoveEvent(QMouseEvent *event);
    void mouseReleaseEvent(QMouseEvent *event);

	void updateOrbitPeriod();
    void updateOrbitRadii();
    void updateCameraPosition(const int angleH, const int angleV, const float zoom);
    void resetParameters();
    void updatePulseProfile();

    QTimer m_frameTimer;

    GLUquadricObj *m_quadricCompanionOrbitPlane;
    GLUquadricObj *m_quadricCompanion;
    GLUquadricObj *m_quadricPulsarOrbitPlane;
	GLUquadricObj *m_quadricPulsar;
	GLUquadricObj *m_quadricPulsarCone1;
	GLUquadricObj *m_quadricPulsarCone2;
	GLUquadricObj *m_quadricPulsarSpinAxis;
	GLUquadricObj *m_quadricPulsarMagneticAxis;

	GLuint m_pulsarTexture;
	GLuint m_backgroundTexture;

	int m_framesPerSecond;

	float m_pulsarRotationAngle;
	float m_pulsarRotationDelta;

	double m_orbitalPeriod;
	float m_orbitRotationAngle;
	float m_orbitRotationDelta;

	float m_pulsarMass;
	double m_pulsarSemiMajorAxis;
	float m_companionMass;
	double m_companionSemiMajorAxis;

	int m_pulsarSpinAxisInclination;
	int m_pulsarMagneticAxisInclination;

	bool m_showOrbits;
	bool m_showRotationAxes;
	bool m_cameraInteraction;

	int m_mouseLastX;
	int m_mouseLastY;
	int m_mouseAngleH;
	int m_mouseAngleV;
	float m_cameraZoom;
	float m_cameraZoomLBound;
	float m_cameraPosX;
	float m_cameraPosY;
	float m_cameraPosZ;

	QVector<float> m_pulseProfile;

	static const float deg2rad;
};

#endif /* PULSARANIMATIONWIDGET_H_ */
